class Forklift {
  constructor(x, y) {
    this.positoin = createVector(x, y);
    this.speed = 5;
    this.path = [];
    this.currentTarget = '';
    this.targetId = 0;
    this.velocity = createVector(0, 0);
    this.direction = createVector(0, 0);
    this.end = false;
    this.currentSection = 0;
    this.cargo = null;
    this.queue = new PriorityQueue();
    this.busy = false;
  }

  draw() {
    fill(225, 255, 0);
    ellipse(this.positoin.x, this.positoin.y, 20);
  }

  setPath(path) {
    this.end = false;
    this.path = path;
    this.currentTarget = this.path[this.targetId];
    this.setVelocity();
  }

  nextTarget() {
    if (this.targetId < this.path.length - 1) {
      this.targetId += 1;
      this.currentTarget = this.path[this.targetId];
    } else {
      // Final destination reached
      magazineState[this.currentTarget].push(this.cargo);
      //this.removeCargo();
      this.end = true;
      this.targetId = 0;
    }
  }

  targetReached() {
    this.currentSection = this.currentTarget;
    return this.end;
  }

  setVelocity() {
    this.direction = this.sub(sections[this.currentTarget], this.positoin);
    this.velocity = this.direction.setMag(this.speed);
  }

  move() {
    this.positoin = this.add(this.positoin, this.velocity);
    if (
      Math.abs(this.positoin.x - sections[this.currentTarget].x) <=
        this.speed &&
      Math.abs(this.positoin.y - sections[this.currentTarget].y) <= this.speed
    ) {
      this.positoin = sections[this.currentTarget];
      this.nextTarget();
      this.setVelocity();
    }
  }

  findPath(targetSection) {
    let data = {
      graph: magazineToGraph(),
      start_node: forklift.currentSection,
      dest_node: int(targetSection),
    };
    httpPost(
      serverUrl + '/shortestPath',
      data,
      response => {
        console.log('Find path request data: ', data);
        let path = response.split('').map(Number);
        this.currentTarget = path[0];
        this.setPath(path);
        going = true;
      },
      error => {
        console.log(error);
      },
    );
  }

  completeOrder(order) {
    this.waitUntilJobIsDone()
      .then(() => {
        if (this.currentSection != order.from) {
          this.findPath(order.from);
        }
      })
      .then(() => this.waitUntilJobIsDone())
      .then(() => {
        this.setCargo(order.what);
        magazineState[order.from] = magazineState[order.from].filter(flower => {
          return flower != order.what;
        });
        this.findPath(order.to);
      })
      .then(() => this.waitUntilJobIsDone())
      .then(() => {
        magazineState[order.to].push(order.what);
        order.what.setMaxMaturity(5 + 2 * order.to);
        console.log(order.message);
        this.busy = false;
      });
  }

  waitUntilJobIsDone() {
    return new Promise(resolve => {
      let interval = setInterval(() => {
        if (!going) {
          clearInterval(interval);
          resolve();
        }
      }, 50);
    });
  }

  completeOrders() {
    while (!this.queue.isEmpty()) {
      let order = queue.dequeue();
      this.completeOrder(order);
    }
  }

  setCargo(cargo) {
    this.cargo = cargo;
  }

  removeCargo() {
    this.cargo = null;
  }

  sub(target, pos) {
    console.log(target, pos);
    return createVector(target.x - pos.x, target.y - pos.y);
  }

  add(target, pos) {
    return createVector(target.x + pos.x, target.y + pos.y);
  }
}
