const serverUrl = 'http://localhost:8000';
let sections;
let roads;
let packageClaim;
let going = false;
let forklift;
let magazineState = { 0: [], 1: [], 2: [], 3: [], 4: [], 5: [], 6: [] };

// This runs once at start
function setup() {
  createCanvas(600, 600).parent('canvas');

  frameRate(30);

  createMagazineLayout();

  select('#button').mousePressed(addOrder());
  sepalWidth = select('#sepalWidth');
  sepalLength = select('#sepalLength');
  petalWidth = select('#petalWidth');
  petalLength = select('#petalLength');

  // Create a forklift instance
  forklift = new Forklift(sections[0].x, sections[0].y);
  setInterval(ageFlowers, 5000);

  select('#completeOrdersButton').mousePressed(forklift.completeOrders());
}

// This runs every frame
function draw() {
  background(64);
  drawMagazine();
  forklift.draw();
  if (going) {
    if (forklift.targetReached()) {
      going = false;
    } else {
      forklift.move();
    }
  }
}

function drawMagazine() {
  noFill();
  stroke(220);
  strokeWeight(10);
  // Draw all the roads in the magazine
  for (let road of roads) {
    line(
      sections[road[0]].x,
      sections[road[0]].y,
      sections[road[1]].x,
      sections[road[1]].y,
    );
  }
  noStroke();
  textAlign(CENTER, CENTER);
  // Draw all sections in the magazine
  for (let section of Object.keys(sections)) {
    if (section === 0) {
      fill(80);
      rect(sections[section].x - 15, sections[section].y, 30, 30);
    } else {
      fill(30);
      ellipse(sections[section].x, sections[section].y, 30);
      fill(255);
      text(section, sections[section].x, sections[section].y);
    }
  }
}

function addOrder() {
  let sw = select('#sepalWidth').value();
  let sl = select('#sepalLength').value();
  let pw = select('#petalWidth').value();
  let pl = select('#petalLength').value();
  let data = {
    sepalWidth: sw,
    sepalLength: sl,
    petalWidth: pw,
    petalLength: pl,
  };
  console.log(data);
  httpPost(serverUrl + '/classify', data, response => {
    magazineState[0].push(new Flower(response));
    forklift.completeOrder(
      new Order(0, magazineState[0][0], magazineState[0][0].type, 'msg'),
    );
  });
}

function createMagazineLayout() {
  unit = width / 6;

  sections = {
    0: { x: 2 * unit, y: unit },
    1: createVector(unit, 2 * unit),
    2: createVector(unit, 3 * unit),
    3: createVector(unit, 4 * unit),
    4: createVector(3 * unit, 2 * unit),
    5: createVector(3 * unit, 3 * unit),
    6: createVector(3 * unit, 4 * unit),
  };
  roads = [
    [1, 5],
    [2, 3],
    [0, 1],
    [0, 4],
    [4, 5],
    [5, 6],
    [3, 6],
    [4, 2],
    [5, 3],
  ];
}

function magazineToGraph() {
  graph = {};
  for (let key of Object.keys(sections)) {
    graph[key] = {};
  }
  for (let road of roads) {
    start = road[0];
    end = road[1];
    graph[start][end] = Math.sqrt(
      Math.pow(sections[start].x - sections[end].x, 2) +
        Math.pow(sections[start].y - sections[end].y, 2),
    );
    graph[end][start] = Math.sqrt(
      Math.pow(sections[start].x - sections[end].x, 2) +
        Math.pow(sections[start].y - sections[end].y, 2),
    );
  }
  return graph;
}

function ageFlowers() {
  for (let key of Object.keys(magazineState)) {
    for (let flower of magazineState[key]) {
      flower.grow();
      if (flower.isReady() && !flower.isOrdered()) {
        let order;
        let priority;
        if (int(key) == 0) {
          order = new Order(0, flower, int(flower), 'Delivering new flower');
          priority = 1;
        } else if (int(key) > 3) {
          order = new Order(int(key), flower, 0, 'Fully grown flower ready');
          priority = 3;
        } else {
          order = new Order(
            int(key),
            flower,
            int(key) + 3,
            'Moving grown flower seedling',
          );
          priority = 2;
        }
        forklift.queue.enqueue([order, priority]);
        flower.ordered = true;
      }
    }
  }
}
